/*
 * GridViewItemProvider.cpp
 *
 *  Created on: Aug 20, 2013
 *      Author: jweller
 */

#include "listviewitemprovider.h"

#include <bb/cascades/controls/container.h>

#include <bb/cascades/controls/label.h>
#include <bb/cascades/controls/imageview.h>

#include <QDebug>

ListViewItemProvider::ListViewItemProvider(const QString &type, const QString& qmlasset,const listitem_callback& callback ) {

    addType(type,qmlasset,callback);
}

ListViewItemProvider::~ListViewItemProvider() {

#ifndef USE_BOOST
	QMap<QString, listitem_callback>::iterator it = callback_map.begin(), end = callback_map.end();
	for(;it != end; ++it)
		delete it.value();
#endif
}

void ListViewItemProvider::addType(const QString& type, const QString& qmlasset, const listitem_callback& callback)
{
    bb::cascades::QmlDocument* doc = bb::cascades::QmlDocument::create(qmlasset);
    if(!doc->hasErrors())
    {
        doc->setParent(this);
        type_map.insert(type,doc);
        callback_map.insert(type,callback);
    }//TODO add additional error checking & handling
}

bb::cascades::VisualNode* ListViewItemProvider::createItem(bb::cascades::ListView* listview,const QString& type)
{
	if(type_map.find(type)!=type_map.end())
	{
		bb::cascades::Container* node = type_map[type]->createRootObject<bb::cascades::Container>();
		return node;
    }
    Q_ASSERT_X(false,__FUNCTION__,type +" TYPE not handled");
	bb::cascades::Container* con = new bb::cascades::Container(0);
	bb::cascades::Label* label = new bb::cascades::Label(con);
	label->setText("ERROR");
	return con;
}

void ListViewItemProvider::updateItem(bb::cascades::ListView* listview,bb::cascades::VisualNode* node,const QString& type,const QVariantList& indexPath, const QVariant& data)
{
	if(callback_map.find(type)!=callback_map.end())
	{
#ifndef USE_BOOST
		(*callback_map[type])(node,indexPath,data);
#else
		callback_map[type](node,indexPath,data);
#endif
	}
	/*bb::cascades::ImageView* image = node->findChild<bb::cascades::ImageView*>("imageview");
	QVariantMap map = data.toMap();
	image->setImageSource(map["thumb"].toUrl());*/
}
