/*
 * GridViewItemProvider.h
 *
 *  Created on: Aug 20, 2013
 *      Author: jweller
 */

#ifndef GRIDVIEWITEMPROVIDER_H_
#define GRIDVIEWITEMPROVIDER_H_
#include <bb/cascades/ListItemProvider>
#include <bb/cascades/QmlDocument>

#include <QMap>

#define USE_BOOST 1

#ifdef USE_BOOST
#define BOOST_STRICT_CONFIG 1
#include <boost/function.hpp>
typedef boost::function<void(bb::cascades::VisualNode*,const QVariantList&, const QVariant&)> listitem_callback;

#else

class item_callback
{
	void (*callback)(bb::cascades::VisualNode* ,const QVariantList& , const QVariant& );
public:
	item_callback(void (*callback)(bb::cascades::VisualNode* ,const QVariantList& , const QVariant& )):callback(callback)
	{

	}
	virtual ~item_callback(){}
	void operator()(bb::cascades::VisualNode* node,const QVariantList& indexPath, const QVariant& data)
	{
		doCall(node,indexPath,data);
	}
protected:
	item_callback(){}
	virtual void doCall(bb::cascades::VisualNode* node,const QVariantList& indexPath, const QVariant& data)
	{
		(*callback)(node,indexPath,data);
	}
};

template<class T>
class memfn_callback : public item_callback
{
	T* t;
	void (T::*callback)(bb::cascades::VisualNode* ,const QVariantList& , const QVariant& );
public:
	memfn_callback(T*t, void (T::*callback)(bb::cascades::VisualNode* ,const QVariantList& , const QVariant& )):t(t),callback(callback)
	{

	}
protected:
	void doCall(bb::cascades::VisualNode* node,const QVariantList& indexPath, const QVariant& data)
	{
		(t->*callback)(node,indexPath,data);
	}
};

typedef item_callback* listitem_callback;
#endif
class ListViewItemProvider: public bb::cascades::ListItemProvider {
	QMap<QString, bb::cascades::QmlDocument*> type_map;
	QMap<QString, listitem_callback> callback_map;
public:
	//ListViewItemProvider(const QString &type, bb::cascades::QmlDocument* doc);
	ListViewItemProvider(const QString &type, const QString& qmlasset,const listitem_callback& callback );
	virtual ~ListViewItemProvider();
	bb::cascades::VisualNode* createItem(bb::cascades::ListView* listview,const QString& type);
	void updateItem(bb::cascades::ListView* listview,bb::cascades::VisualNode* node,const QString& type,const QVariantList& indexPath, const QVariant& data);
    void addType(const QString& type, const QString& qmlasset, const listitem_callback& callback);

};

#endif /* GRIDVIEWITEMPROVIDER_H_ */
